/**
 * plugin.min.js
 *
 * Released under Attribution-ShareAlike 4.0 International License.
 * Author: Ignacio Gros (http://gros.es/) for http://exelearning.net/
 *
 * License: http://creativecommons.org/licenses/by-sa/4.0/
 */

/**
 * Insert ABC Music Notation Plugin for TinyMCE
 *
 * To be used with export/*.*
 *
	* acoustic_grand_piano-ogg.js, by Roberto Gordo Saez, under the CC BY license.
	* abcjs.js, by Paul Rosen and Gregory Dyke, under the GNU GPL v3 license.
	* midi.js, by Michael Deal, under the MIT License.
	* Scribe.abcMusic, by Ignacio Gros, under the CC BY-SA license.
 *
 */
 
tinymce.PluginManager.requireLangPack('abcmusic');
 
tinymce.PluginManager.add('abcmusic', function(editor, url) {
	
	ABCmusicDialog = {
		
		cssClass : "abc-music",
		
		jQueryPath : "/scripts/exe_jquery.js",		

		abcjsPath : "/scripts/tinymce_4/js/tinymce/plugins/abcmusic/export/exe_abcmusic.js",
		
		abccssPath : "/scripts/tinymce_4/js/tinymce/plugins/abcmusic/export/exe_abcmusic.css",
		
		soundFontPath : "/scripts/tinymce_4/js/tinymce/plugins/abcmusic/export/",
		
		// Load the previous values (if needed)
		getValues : function(){
			
			ABCmusicDialog.inPRE = false;
			ABCmusicDialog.currentPRE = "";

			var abcInstructions = jQuery("#abcInstructions");
			abcInstructions.html(abcInstructions.html().replace("eXeABC","<a href='http://abcnotation.com' target='_blank' style='text-decoration:underline;cursor:pointer'>abcnotation.com</a>"));			
			
			var n = editor.selection.getNode();
			
			if (n.nodeName=="PRE") {
				ABCmusicDialog.inPRE = true;
				ABCmusicDialog.currentPRE = n;
				if (jQuery(n).hasClass(ABCmusicDialog.cssClass+"-animated")) win.find("#animation").checked(true);
				if (jQuery(n).hasClass(ABCmusicDialog.cssClass+"-midi")) win.find("#midi").checked(true);
				win.find('#htmlSource')[0].value(n.innerHTML);
			}
			
		},
		
		activateButton : function(node) {
			
			if (node.nodeName=="PRE" && node.className.indexOf(ABCmusicDialog.cssClass)==0) return true;
			return false;
			
		},

		openDialog : function(){

			// Open the window
			win = editor.windowManager.open({
				title: tinymce.translate('ABC Music Notation'),
				// width: 500,
				body: [
					{
						type: 'container',
						layout: 'flex',
						direction: 'row',
						align: 'stretch',
						padding: 5,
						spacing: 10,
						items: [
							{
								id: 'animation', 
								name: 'animation', 
								type: 'checkbox', 
								checked: false, 
								text: tinymce.translate('Animation'),
                                hidden: true
							},
							{
								id: 'midi', 
								name: 'midi', 
								type: 'checkbox', 
								checked: false, 
								text: 'MIDI'
							}
						]
					},				
					{
						type: 'label',
						text: tinymce.translate('Paste or type a valid ABC code:'),
						forId: 'htmlSource'
					},
					{
						id: 'htmlSource',
						type: 'textbox',
						name: 'htmlSource',
						minWidth: 500,
						minHeight: 150,
						// value: ...(),
						multiline: true
					},
					{
						type   : 'button',
						name   : 'button',
						text   : tinymce.translate("Preview"),
						onclick : function(){
							ABCmusicDialog.openPopup();
						}
					},					
					{
						id: 'abcInstructions',
						type: 'label',
						text: tinymce.translate('Visit eXeABC for help, tunes, ABC software...')
					},
					{
						type: 'label',
						text: tinymce.translate('ePub export warning: This might not work in some ePub readers.')
					}					
				],
				onsubmit: function(e) {
					// Insert content when the window form is submitted
					var content = e.data.htmlSource;
					
					if (content=="") return "";
					
					var css = ABCmusicDialog.cssClass;
					
					var animation = win.find("#animation").checked();
					var animatedCSS = ABCmusicDialog.cssClass+"-animated"
					
					var midi = win.find("#midi").checked();
					var midiCSS = ABCmusicDialog.cssClass+"-midi"				
					
					if (!ABCmusicDialog.inPRE) {
						
						if (animation) css += " "+animatedCSS;
						if (midi) css += " "+midiCSS;
						content = '<pre class="'+css+'">'+content+'</pre><p>...</p>';
						editor.execCommand('mceInsertContent', false, content);
						
					} else {
						
						var n = jQuery(ABCmusicDialog.currentPRE);
						n.addClass(css);
						
						if (animation) n.addClass(animatedCSS);
						else n.removeClass(animatedCSS);
						
						if (midi) n.addClass(midiCSS);
						else n.removeClass(midiCSS);					
						
						editor.dom.setHTML(ABCmusicDialog.currentPRE, content);
						
					}

				}
			});

			ABCmusicDialog.getValues();
		
		},
		
		openPopup : function(){
			var w = 830;
			var h = 550;
			var win = null;
			var winl = (screen.width-w)/2;
			var wint = (screen.height-h)/2;
			if (winl < 0) winl = 0;
			if (wint < 0) wint = 0;
			var settings = 'height=' + h + ',';
			settings += 'width=' + w + ',';
			settings += 'top=' + wint + ',';
			settings += 'left=' + winl + ',';
			// settings += features;
			win = window.open(url + "/abcmusic.html","ABCmusicPopup",settings);
			// This line is required in all plugins with i18n
			win.$exe_i18n=$exe_i18n;
			win.window.focus();	
		}		
		
	} // ABCmusicDialog	
	
	editor.addButton('abcmusic', {
		image: url + '/img/abcmusic.png',
		tooltip: tinymce.translate('ABC Music Notation'),
		onclick: ABCmusicDialog.openDialog,
		onPostRender: function() {
			var ctrl = this;
			editor.on('NodeChange', function(e) {
				ctrl.active(ABCmusicDialog.activateButton(e.element));
			});
		}
	});	
	
	editor.addMenuItem('abcmusic', {
		// icon: 'image',
		text: tinymce.translate('ABC Music Notation'),
		onclick: ABCmusicDialog.openDialog,
		context: 'insert'
	});
	
	editor.on('init', function(e) {
		editor.dom.loadCSS(url + "/css/content.css");
		
		// jQuery is required
		if (typeof(jQuery)!='function') {
			alert(tinymce.translate('ABC Music Notation')+": "+tinymce.translate('jQuery is required'));
			return false;			
		}
		
		// i18n in CSS (type names in the right language)
		var d = editor.getDoc();
		var s = '<style type="text/css">';
			s += '.'+ABCmusicDialog.cssClass+':before{content:"'+tinymce.translate('ABC Music')+'"}';
			s += '.'+ABCmusicDialog.cssClass+'-animated:before{content:"'+tinymce.translate('ABC Music')+' + '+tinymce.translate('Animation')+'"}';
			s += '.'+ABCmusicDialog.cssClass+'-midi:before{content:"'+tinymce.translate('ABC Music')+' + MIDI"}';
			s += '.'+ABCmusicDialog.cssClass+'-animated.'+ABCmusicDialog.cssClass+'-midi:before{content:"'+tinymce.translate('ABC Music')+' + '+tinymce.translate('Animation')+' + MIDI"}';
		s += '</style>';
		jQuery("HEAD",d).append(s);
		
	});
	
});