// The New eXeLearning: Compare this file with advlist/plugin.js (TinyMCE Dev Package)
// This plugin just adds a new OL type: "Automatic Numbering"
// See eXe's base.css file to see what .auto-numbered does

/**
 * plugin.js
 *
 * Released under LGPL License.
 * Copyright (c) 1999-2015 Ephox Corp. All rights reserved
 *
 * License: http://www.tinymce.com/license
 * Contributing: http://www.tinymce.com/contributing
 */

/*global tinymce:true */

tinymce.PluginManager.add('exelist', function(editor) {
	var olMenuItems, ulMenuItems, lastStyles = {};

	function buildMenuItems(listName, styleValues) {
		var items = [];

		tinymce.each(styleValues.split(/[ ,]/), function(styleValue) {
			items.push({
				text: styleValue.replace(/\-/g, ' ').replace(/\b\w/g, function(chr) {
					return chr.toUpperCase();
				}),
				data: styleValue == 'default' ? '' : styleValue
			});
		});

		return items;
	}

	// The New eXeLearning
	olMenuItems = [
		{ "text": _("Default"), "data": "" },
		{ "text": _("Lower Alpha"), "data": "lower-alpha" },
		{ "text": _("Lower Greek"), "data": "lower-greek" },
		{ "text": _("Lower Roman"), "data": "lower-roman" },
		{ "text": _("Upper Alpha"), "data": "upper-alpha" },
		{ "text": _("Upper Roman"), "data": "upper-roman" },
		{ "text": _("Automatic Numbering"), "data": "decimal" }
	],	
	// / The New eXeLearning

	ulMenuItems = buildMenuItems('UL', editor.getParam("exelist_bullet_styles", "default,circle,disc,square"));

	function applyListFormat(listName, styleValue) {
		editor.undoManager.transact(function() {
			var list, dom = editor.dom, sel = editor.selection;

			// Check for existing list element
			list = dom.getParent(sel.getNode(), 'ol,ul');

			// Switch/add list type if needed
			if (!list || list.nodeName != listName || styleValue === false) {
				editor.execCommand(listName == 'UL' ? 'InsertUnorderedList' : 'InsertOrderedList');
			}

			// Set style
			styleValue = styleValue === false ? lastStyles[listName] : styleValue;
			lastStyles[listName] = styleValue;

			list = dom.getParent(sel.getNode(), 'ol,ul');
			if (list) {
				
				// The New eXeLearning
				
				// Get the list current CSS class and check if the "auto-numbered" class has to be added
				var currentClass = dom.getAttrib(list, "class");
				var newClass = "";
				var classToAdd = "";
				if (styleValue=="decimal") {
					classToAdd = " auto-numbered";
				}					
				
				// Get the final CSS class
				var currentClasses = currentClass.split(" ");
				var l = currentClasses.length;
				for (var i=0;i<l;i++) {
					var c = currentClasses[i];
					if (c!="" && c!="auto-numbered") {
						newClass += c;
						if (i<(l-1)) newClass += " ";
					}
				}
				
				newClass += classToAdd;
				dom.setAttrib(list, "class", newClass);
				
				// /The New eXeLearning
				
				dom.setStyle(list, 'listStyleType', styleValue ? styleValue : null);
				list.removeAttribute('data-mce-style');
			}

			editor.focus();
		});
	}

	function updateSelection(e) {
		var listStyleType = editor.dom.getStyle(editor.dom.getParent(editor.selection.getNode(), 'ol,ul'), 'listStyleType') || '';

		e.control.items().each(function(ctrl) {
			ctrl.active(ctrl.settings.data === listStyleType);
		});
	}

	editor.addButton('numlist', {
		type: 'splitbutton',
		tooltip: 'Numbered list',
		menu: olMenuItems,
		onshow: updateSelection,
		onselect: function(e) {
			applyListFormat('OL', e.control.settings.data);
		},
		onclick: function() {
			applyListFormat('OL', false);
		}
	});

	editor.addButton('bullist', {
		type: 'splitbutton',
		tooltip: 'Bullet list',
		menu: ulMenuItems,
		onshow: updateSelection,
		onselect: function(e) {
			applyListFormat('UL', e.control.settings.data);
		},
		onclick: function() {
			applyListFormat('UL', false);
		}
	});
});